@@ -4,7 +4,7 @@
     
     Copyright (C) 1999 David A. Hinds -- dahinds@users.sourceforge.net
 
-    3c589_cs.c 1.163 2002/02/17 23:30:20
+    3c589_cs.c 1.166 2002/10/22 02:11:06
 
     The network driver code is based on Donald Becker's 3c589 code:
     
@@ -17,9 +17,6 @@
 
 ======================================================================*/
 
-#include <pcmcia/config.h>
-#include <pcmcia/k_compat.h>
-
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/kernel.h>
@@ -137,7 +134,7 @@ MODULE_PARM(irq_list, "1-4i");
 INT_MODULE_PARM(pc_debug, PCMCIA_DEBUG);
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"3c589_cs.c 1.163 2002/02/17 23:30:20 (David Hinds)";
+"3c589_cs.c 1.166 2002/10/22 02:11:06 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -413,7 +410,6 @@ static void tc589_config(dev_link_t *link)
 
     copy_dev_name(lp->node, dev);
     link->dev = &lp->node;
-    link->state &= ~DEV_CONFIG_PENDING;
     
     /* The address and resource configuration register aren't loaded from
        the EEPROM and *must* be set to 0 and IRQ3 for the PCMCIA version. */
@@ -434,12 +430,14 @@ static void tc589_config(dev_link_t *link)
     printk(KERN_INFO "  %dK FIFO split %s Rx:Tx, %s xcvr\n",
 	   (i & 7) ? 32 : 8, ram_split[(i >> 16) & 3],
 	   if_names[dev->if_port]);
+    link->state &= ~DEV_CONFIG_PENDING;
     return;
 
 cs_failed:
     cs_error(link->handle, last_fn, last_ret);
 failed:
     tc589_release((u_long)link);
+    link->state &= ~DEV_CONFIG_PENDING;
     return;
     
 } /* tc589_config */
@@ -738,7 +736,7 @@ static int el3_start_xmit(struct sk_buff *skb, struct net_device *dev)
     outw(skb->len, ioaddr + TX_FIFO);
     outw(0x00, ioaddr + TX_FIFO);
     /* ... and the packet rounded to a doubleword. */
-    outsl_ns(ioaddr + TX_FIFO, skb->data, (skb->len + 3) >> 2);
+    outsl(ioaddr + TX_FIFO, skb->data, (skb->len + 3) >> 2);
 
     dev->trans_start = jiffies;
     if (inw(ioaddr + TX_FREE) > 1536) {
@@ -997,8 +995,8 @@ static int el3_rx(struct net_device *dev)
 	    if (skb != NULL) {
 		skb->dev = dev;
 		skb_reserve(skb, 2);
-		insl_ns(ioaddr+RX_FIFO, skb_put(skb, pkt_len),
-			(pkt_len+3)>>2);
+		insl(ioaddr+RX_FIFO, skb_put(skb, pkt_len),
+		     (pkt_len+3)>>2);
 		skb->protocol = eth_type_trans(skb, dev);
 		netif_rx(skb);
 		dev->last_rx = jiffies;
