@@ -11,8 +11,8 @@
 
     Copyright (C) 2001 David A. Hinds -- dahinds@users.sourceforge.net
 
-    axnet_cs.c 1.26 2002/02/17 23:30:21
-    
+    axnet_cs.c 1.30 2002/10/22 02:11:43
+
     The network driver code is based on Donald Becker's NE2000 code:
 
     Written 1992,1993 by Donald Becker.
@@ -24,9 +24,6 @@
 
 ======================================================================*/
 
-#include <pcmcia/config.h>
-#include <pcmcia/k_compat.h>
-
 #include <linux/kernel.h>
 #include <linux/module.h>
 #include <linux/init.h>
@@ -40,6 +37,7 @@
 #include <asm/io.h>
 #include <asm/system.h>
 #include <asm/byteorder.h>
+#include <asm/uaccess.h>
 
 #include <linux/netdevice.h>
 #include "ax8390.h"
@@ -86,7 +84,7 @@ MODULE_PARM(irq_list, "1-4i");
 INT_MODULE_PARM(pc_debug, PCMCIA_DEBUG);
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"axnet_cs.c 1.26 2002/02/17 23:30:21 (David Hinds)";
+"axnet_cs.c 1.30 2002/10/22 02:11:43 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -484,7 +482,6 @@ static void axnet_config(dev_link_t *link)
 
     copy_dev_name(info->node, dev);
     link->dev = &info->node;
-    link->state &= ~DEV_CONFIG_PENDING;
 
     if (inb(dev->base_addr + AXNET_TEST) != 0)
 	info->flags |= IS_AX88790;
@@ -492,8 +489,8 @@ static void axnet_config(dev_link_t *link)
 	info->flags |= IS_AX88190;
 
     printk(KERN_INFO "%s: Asix AX88%d90: io %#3lx, irq %d, hw_addr ",
-		dev->name, ((info->flags & IS_AX88790) ? 7 : 1),
-		dev->base_addr, dev->irq);
+	   dev->name, ((info->flags & IS_AX88790) ? 7 : 1),
+	   dev->base_addr, dev->irq);
     for (i = 0; i < 6; i++)
 	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
 
@@ -524,12 +521,14 @@ static void axnet_config(dev_link_t *link)
 	printk(KERN_NOTICE "  No MII transceivers found!\n");
     }
 
+    link->state &= ~DEV_CONFIG_PENDING;
     return;
 
 cs_failed:
     cs_error(link->handle, last_fn, last_ret);
 failed:
     axnet_release((u_long)link);
+    link->state &= ~DEV_CONFIG_PENDING;
     return;
 } /* axnet_config */
 
@@ -588,7 +587,7 @@ static int axnet_event(event_t event, int priority,
 	}
 	break;
     case CS_EVENT_CARD_INSERTION:
-	link->state |= DEV_PRESENT;
+	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
 	axnet_config(link);
 	break;
     case CS_EVENT_PM_SUSPEND:
@@ -859,8 +858,8 @@ static void get_8390_hdr(struct net_device *dev,
     outb_p(ring_page, nic_base + EN0_RSARHI);
     outb_p(E8390_RREAD+E8390_START, nic_base + AXNET_CMD);
 
-    insw_ns(nic_base + AXNET_DATAPORT, hdr,
-	    sizeof(struct e8390_pkt_hdr)>>1);
+    insw(nic_base + AXNET_DATAPORT, hdr,
+	 sizeof(struct e8390_pkt_hdr)>>1);
     /* Fix for big endian systems */
     hdr->count = le16_to_cpu(hdr->count);
 
@@ -883,7 +882,7 @@ static void block_input(struct net_device *dev, int count,
     outb_p(ring_offset >> 8, nic_base + EN0_RSARHI);
     outb_p(E8390_RREAD+E8390_START, nic_base + AXNET_CMD);
 
-    insw_ns(nic_base + AXNET_DATAPORT,buf,count>>1);
+    insw(nic_base + AXNET_DATAPORT,buf,count>>1);
     if (count & 0x01)
 	buf[count-1] = inb(nic_base + AXNET_DATAPORT), xfer_count++;
 
@@ -910,7 +909,7 @@ static void block_output(struct net_device *dev, int count,
     outb_p(0x00, nic_base + EN0_RSARLO);
     outb_p(start_page, nic_base + EN0_RSARHI);
     outb_p(E8390_RWRITE+E8390_START, nic_base + AXNET_CMD);
-    outsw_ns(nic_base + AXNET_DATAPORT, buf, count>>1);
+    outsw(nic_base + AXNET_DATAPORT, buf, count>>1);
 }
 
 /*====================================================================*/
@@ -966,7 +965,6 @@ module_exit(exit_axnet_cs);
   a simple innocent change. Please contact me or Donald if you think
   you have found something that needs changing. -- PG
 
-
   Changelog:
 
   Paul Gortmaker	: remove set_bit lock, other cleanups.
@@ -1071,8 +1069,6 @@ static void do_set_multicast_list(struct net_device *dev);
  *	them.
  */
  
-
-
 /**
  * ax_open - Open/initialize the board.
  * @dev: network device to initialize
@@ -1128,17 +1124,17 @@ static int ax_open(struct net_device *dev)
  */
 int ax_close(struct net_device *dev)
 {
-        unsigned long flags;
+	unsigned long flags;
 
-        /*
-         *      Hold the page lock during close
-         */
+	/*
+	 *      Hold the page lock during close
+	 */
 
 	spin_lock_irqsave(&dev_lock(dev), flags);
-        AX88190_init(dev, 0);
+	AX88190_init(dev, 0);
 	spin_unlock_irqrestore(&dev_lock(dev), flags);
-        netif_stop_queue(dev);
-        return 0;
+	netif_stop_queue(dev);
+	return 0;
 }
 
 /**
@@ -1216,7 +1212,6 @@ static int ei_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	outb_p(0x00, e8390_base + EN0_IMR);
 	spin_unlock_irqrestore(&ei_local->page_lock, flags);
 	
-	
 	/*
 	 *	Slow phase with lock held.
 	 */
@@ -1309,7 +1304,7 @@ static int ei_start_xmit(struct sk_buff *skb, struct net_device *dev)
     
 	return 0;
 }
-
+
 /**
  * ax_interrupt - handle the interrupts from an 8390
  * @irq: interrupt number
@@ -1840,8 +1835,6 @@ static int axdev_init(struct net_device *dev)
         
 	return 0;
 }
-
-
 
 /* This page of functions should be 8390 generic */
 /* Follow National Semi's recommendations for initializing the "NIC". */
