@@ -52,9 +52,6 @@
 
 ======================================================================*/
 
-#include <pcmcia/config.h>
-#include <pcmcia/k_compat.h>
-
 #include <linux/kernel.h>
 #include <linux/init.h>
 #include <linux/sched.h>
@@ -377,7 +374,7 @@ static void ibmtr_config(dev_link_t *link)
     mem.CardOffset = (TR_OLD ? req.Base : mmiobase);
     mem.Page = 0;
     CS_CHECK(MapMemPage, link->win, &mem);
-    ti->mmio = (u_long)ioremap(req.Base, req.Size);
+    (void *)(ti->mmio) = ioremap(req.Base, req.Size);
 
     /* Allocate the SRAM memory window */
     req.Attributes = WIN_DATA_WIDTH_16|WIN_MEMORY_TYPE_CM|WIN_ENABLE;
@@ -414,7 +411,6 @@ static void ibmtr_config(dev_link_t *link)
     }
 
     link->dev = &info->node;
-    link->state &= ~DEV_CONFIG_PENDING;
 
     printk(KERN_INFO "%s: port %#3lx, irq %d,",
            dev->name, dev->base_addr, dev->irq);
@@ -424,12 +420,14 @@ static void ibmtr_config(dev_link_t *link)
     for (i = 0; i < TR_ALEN; i++)
         printk("%02X", dev->dev_addr[i]);
     printk("\n");
+    link->state &= ~DEV_CONFIG_PENDING;
     return;
 
 cs_failed:
     cs_error(link->handle, last_fn, last_ret);
 failed:
     ibmtr_release((u_long)link);
+    link->state &= ~DEV_CONFIG_PENDING;
 } /* ibmtr_config */
 
 /*======================================================================
@@ -503,7 +501,7 @@ static int ibmtr_event(event_t event, int priority,
         }
         break;
     case CS_EVENT_CARD_INSERTION:
-        link->state |= DEV_PRESENT;
+        link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
 	ibmtr_config(link);
 	break;
     case CS_EVENT_PM_SUSPEND:
