@@ -3,7 +3,7 @@
     Device driver for Intel 82365 and compatible PC Card controllers,
     and Yenta-compatible PCI-to-CardBus controllers.
 
-    i82365.c 1.347 2002/02/26 04:50:28
+    i82365.c 1.356 2003/01/25 06:02:02
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -32,9 +32,6 @@
 
 ======================================================================*/
 
-#include <pcmcia/config.h>
-#include <pcmcia/k_compat.h>
-
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/config.h>
@@ -141,7 +138,7 @@ INT_MODULE_PARM(pci_int, 1);		/* PCI IO card irqs? */
 INT_MODULE_PARM(pc_debug, PCMCIA_DEBUG);
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static const char *version =
-"i82365.c 1.347 2002/02/26 04:50:28 (David Hinds)";
+"i82365.c 1.356 2003/01/25 06:02:02 (David Hinds)";
 #else
 #define DEBUG(n, args...) do { } while (0)
 #endif
@@ -360,10 +357,9 @@ static void i365_set_pair(socket_info_t *s, u_short reg, u_short data)
 
 static int __init get_pci_irq(socket_info_t *s)
 {
-    u8 irq;
-    irq = pci_find_slot(s->bus, s->devfn)->irq;
-    if ((irq == 0) && (pci_csc || pci_int))
-	irq = pci_irq_list[s - socket];
+    u8 irq = pci_irq_list[s - socket];
+    if (!irq)
+	irq = pci_find_slot(s->bus, s->devfn)->irq;
     if (irq >= NR_IRQS) irq = 0;
     s->cap.pci_irq = irq;
     return irq;
@@ -871,7 +867,6 @@ static void topic_set_state(socket_info_t *s)
     topic_state_t *p = &s->state.topic;
     u_int state;
     pci_writeb(s, TOPIC_SLOT_CONTROL, p->slot);
-    pci_writeb(s, TOPIC_SLOT_CONTROL, p->slot);
     pci_writeb(s, TOPIC_CARD_CONTROL, p->ccr);
     pci_writeb(s, TOPIC_CARD_DETECT, p->cdr);
     pci_writel(s, TOPIC_REGISTER_CONTROL, p->rcr);
@@ -1386,7 +1381,7 @@ static int __init is_alive(socket_info_t *s)
 
 /*====================================================================*/
 
-static void __init add_socket(u_short port, int psock, int type)
+static void __init add_socket(u_int port, int psock, int type)
 {
     socket_info_t *s = socket+sockets;
     s->ioaddr = port;
@@ -1521,6 +1516,7 @@ static void __init add_pci_bridge(int type, u_short v, u_short d)
     u_int addr, ns;
 
     pci_enable_device(pci_find_slot(s->bus, s->devfn));
+    pci_writew(s, PCI_COMMAND, CMD_DFLT);
 
     if (type == PCIC_COUNT) type = IS_UNK_PCI;
     pci_readl(s, PCI_BASE_ADDRESS_0, &addr);
@@ -1570,6 +1566,7 @@ static void __init add_cb_bridge(int type, u_short v, u_short d0)
 
 	pci_enable_device(pci_find_slot(bus, devfn));
 	pci_set_power_state(pci_find_slot(bus, devfn), 0);
+	pci_writew(s, PCI_COMMAND, CMD_DFLT);
 
 	/* Set up CardBus register mapping */
 	pci_writel(s, CB_LEGACY_MODE_BASE, 0);
@@ -2069,6 +2066,7 @@ static int i365_get_mem_map(socket_info_t *s, struct pccard_mem_map *mem)
     if (s->type == IS_PD6729) {
 	i365_set(s, PD67_EXT_INDEX, PD67_MEM_PAGE(map));
 	addr = i365_get(s, PD67_EXT_DATA) << 24;
+	mem->sys_stop += addr; mem->sys_start += addr;
     } else if (s->flags & IS_CARDBUS) {
 	addr = i365_get(s, CB_MEM_PAGE(map)) << 24;
 	mem->sys_stop += addr; mem->sys_start += addr;
@@ -2261,8 +2259,8 @@ static int cb_set_bridge(socket_info_t *s, struct cb_bridge_map *m)
 	pci_writel(s, CB_MEM_BASE(map), m->start);
 	pci_writel(s, CB_MEM_LIMIT(map), m->stop);
     } else {
-	pci_writel(s, CB_MEM_BASE(map), 0);
 	pci_writel(s, CB_MEM_LIMIT(map), 0);
+	pci_writel(s, CB_MEM_BASE(map), 0);
     }
     return 0;
 }
@@ -2463,6 +2461,14 @@ static int __init init_i82365(void)
 	return -EINVAL;
     }
     DEBUG(0, "%s\n", version);
+
+#ifdef CONFIG_PCI
+    if (pcic[IS_UNK_CARDBUS].flags != (IS_CARDBUS|IS_UNKNOWN)) {
+	printk(KERN_NOTICE "i82365: bad pcic_id enumeration!\n");
+	return -EINVAL;
+    }
+#endif
+
     printk(KERN_INFO "Intel ISA/PCI/CardBus PCIC probe:\n");
     sockets = 0;
 
