@@ -27,9 +27,6 @@
 ======================================================================*/
 
 #include <linux/config.h>
-#ifdef __IN_PCMCIA_PACKAGE__
-#include <pcmcia/k_compat.h>
-#else
 #ifndef __KERNEL__
 #define __KERNEL__
 #endif
@@ -43,7 +40,6 @@
 #include <linux/modversions.h>
 #endif   
 
-#endif 
 #include <linux/kernel.h>
 #include <linux/module.h>
 
@@ -533,6 +529,7 @@ static void airo_config(dev_link_t *link)
  cs_failed:
 	cs_error(link->handle, last_fn, last_ret);
 	airo_release((u_long)link);
+	link->state &= ~DEV_CONFIG_PENDING;
 	
 } /* airo_config */
 
